package org.sky.system.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import lombok.AllArgsConstructor;
import org.sky.core.boot.ctrl.KdController;
import org.sky.core.cache.utils.CacheUtil;
import org.sky.core.mp.support.Condition;
import org.sky.core.mp.support.Query;
import org.sky.core.secure.KdUser;
import org.sky.core.secure.annotation.IsAdmin;
import org.sky.core.secure.annotation.IsAdministrator;
import org.sky.core.tenant.annotation.NonDS;
import org.sky.core.tool.api.R;
import org.sky.core.tool.constant.KdConstant;
import org.sky.core.tool.support.Kv;
import org.sky.core.tool.utils.Func;
import org.sky.system.pojo.entity.Tenant;
import org.sky.system.pojo.entity.TenantPackage;
import org.sky.system.pojo.vo.TenantVO;
import org.sky.system.service.ITenantPackageService;
import org.sky.system.service.ITenantService;
import org.sky.system.wrapper.TenantWrapper;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

import static org.sky.core.cache.constant.CacheConstant.SYS_CACHE;
import static org.sky.core.tenant.constant.TenantBaseConstant.TENANT_DATASOURCE_CACHE;
import static org.sky.core.tenant.constant.TenantBaseConstant.TENANT_DATASOURCE_EXIST_KEY;
import static org.sky.system.cache.SysCache.TENANT_PACKAGE_ID;
import static org.sky.system.cache.SysCache.TENANT_TENANT_ID;

/**
 * 控制器
 *
 */
@NonDS
//@Hidden
@RestController
@AllArgsConstructor
@RequestMapping("/tenant")
@Tag(name = "租户管理", description = "接口")
public class TenantController extends KdController {

	private final ITenantService tenantService;

	private final ITenantPackageService tenantPackageService;

	/**
	 * 详情
	 */
//	@IsAdmin
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description = "传入tenant")
	public R<Tenant> detail(Tenant tenant) {
		Tenant detail = tenantService.getOne(Condition.getQueryWrapper(tenant));
		return R.data(detail);
	}

	/**
	 * 分页
	 */
//	@IsAdmin
	@GetMapping("/list")
//	@Parameters({
//		@Parameter(name = "tenantId", description = "参数名称", in = ParameterIn.QUERY, schema = @Schema(type = "string")),
//		@Parameter(name = "tenantName", description = "角色别名", in = ParameterIn.QUERY, schema = @Schema(type = "string")),
//		@Parameter(name = "contactNumber", description = "联系电话", in = ParameterIn.QUERY, schema = @Schema(type = "string"))
//	})
//	@ApiOperationSupport(order = 2)
//	@Operation(summary = "分页", description = "传入tenant")
//	public R<IPage<Tenant>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> tenant, Query query, KdUser kdUser) {
//		TenantWrapper.build().entityQuery(tenant);
//		QueryWrapper<Tenant> queryWrapper = Condition.getQueryWrapper(tenant, Tenant.class);
//		IPage<Tenant> pages = tenantService.page(Condition.getPage(query), (!kdUser.getTenantId().equals(KdConstant.ADMIN_TENANT_ID)) ? queryWrapper.lambda().eq(Tenant::getTenantId, kdUser.getTenantId()) : queryWrapper);
//		return R.data(pages);
//	}

	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description = "传入tenant")
	public R<IPage<Tenant>> list(Tenant tenant, Query query ) {
//		TenantWrapper.build().entityQuery(tenant);
//		QueryWrapper<Tenant> queryWrapper = Condition.getQueryWrapper(tenant, Tenant.class);
//		IPage<Tenant> pages = tenantService.page(Condition.getPage(query), (!kdUser.getTenantId().equals(KdConstant.ADMIN_TENANT_ID)) ? queryWrapper.lambda().eq(Tenant::getTenantId, kdUser.getTenantId()) : queryWrapper);
//		return R.data(pages);

		IPage<Tenant> pages = tenantService.selectTenantPage(Condition.getPage(query), tenant);
		return R.data(pages);
	}

	/**
	 * 下拉数据源
	 */
	@GetMapping("/select")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "下拉数据源", description = "传入tenant")
	public R<List<TenantVO>> select(Tenant tenant, KdUser kdUser) {
		QueryWrapper<Tenant> queryWrapper = Condition.getQueryWrapper(tenant);
		List<Tenant> list = tenantService.list((!kdUser.getTenantId().equals(KdConstant.ADMIN_TENANT_ID)) ? queryWrapper.lambda().eq(Tenant::getTenantId, kdUser.getTenantId()) : queryWrapper);
		return R.data(TenantWrapper.build().listVO(list));
	}

	/**
	 * 自定义分页
	 */
//	@IsAdmin
	@GetMapping("/page")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "分页", description = "传入tenant")
	public R<IPage<Tenant>> page(Tenant tenant, Query query) {
		IPage<Tenant> pages = tenantService.selectTenantPage(Condition.getPage(query), tenant);
		return R.data(pages);
	}

	/**
	 * 新增或修改
	 */
//	@IsAdministrator
	@PostMapping("/submit")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "新增或修改", description = "传入tenant")
	public R submit(@Valid @RequestBody Tenant tenant) {
		return R.status(tenantService.submitTenant(tenant));
	}


	@GetMapping("vip-years")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "获取vip年度")
	public R<List<String>> getVipYears(){
		return R.data(tenantService.getVipYears());
	}

	/**
	 * 删除至回收站
	 */
	@IsAdministrator
	@PostMapping("/recycle")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "删除至回收站", description = "传入ids")
	public R recycle(@Parameter(description = "主键集合", required = true) @RequestParam String ids) {
		return R.status(tenantService.recycleTenant(Func.toLongList(ids)));
	}

	/**
	 * 从回收站恢复
	 */
	@IsAdministrator
	@PostMapping("/pass")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "从回收站恢复", description = "传入ids")
	public R pass(@Parameter(description = "主键集合", required = true) @RequestParam String ids) {
		return R.status(tenantService.passTenant(Func.toLongList(ids)));
	}

	/**
	 * 从回收站删除
	 */
	@IsAdministrator
	@PostMapping("/remove")
	@ApiOperationSupport(order = 8)
	@Operation(summary = "从回收站删除", description = "传入ids")
	public R remove(@Parameter(description = "主键集合", required = true) @RequestParam String ids) {
		return R.status(tenantService.removeTenant(Func.toLongList(ids)));
	}

	/**
	 * 授权配置
	 */
	@IsAdministrator
	@PostMapping("/setting")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "授权配置", description = "传入ids,accountNumber,expireTime")
	public R setting(@Parameter(description = "主键集合", required = true) @RequestParam String ids, @Parameter(description = "账号额度") Integer accountNumber, @Parameter(description = "过期时间") Date expireTime) {
		return R.status(tenantService.setting(accountNumber, expireTime, ids));
	}

	/**
	 * 数据源配置
	 */
	@IsAdministrator
	@PostMapping("datasource")
	@ApiOperationSupport(order = 10)
	@Operation(summary = "数据源配置", description = "传入datasource_id")
	public R datasource(@Parameter(description = "租户ID", required = true) @RequestParam String tenantId, @Parameter(description = "数据源ID", required = true) @RequestParam Long datasourceId){
		CacheUtil.evict(TENANT_DATASOURCE_CACHE, TENANT_DATASOURCE_EXIST_KEY, tenantId, Boolean.FALSE);
		return R.status(tenantService.update(Wrappers.<Tenant>update().lambda().set(Tenant::getDatasourceId, datasourceId).eq(Tenant::getTenantId, tenantId)));
	}

	/**
	 * 根据名称查询列表
	 *
	 * @param name 租户名称
	 */
	@IsAdmin
	@GetMapping("/find-by-name")
	@ApiOperationSupport(order = 11)
	@Operation(summary = "详情", description = "传入tenant")
	public R<List<Tenant>> findByName(String name) {
		List<Tenant> list = tenantService.list(Wrappers.<Tenant>query().lambda().like(Tenant::getTenantName, name));
		return R.data(list);
	}

	/**
	 * 根据域名查询信息
	 *
	 * @param domain 域名
	 */
	@GetMapping("/info")
	@ApiOperationSupport(order = 12)
	@Operation(summary = "配置信息", description = "传入domain")
	public R<Kv> info(String domain) {
		Tenant tenant = tenantService.getOne(Wrappers.<Tenant>query().lambda().eq(Tenant::getDomainUrl, domain));
		Kv kv = Kv.create();
		if (tenant != null) {
			kv.set("tenantId", tenant.getTenantId())
				.set("domain", tenant.getDomainUrl())
				.set("backgroundUrl", tenant.getBackgroundUrl());
		}
		return R.data(kv);
	}

	/**
	 * 根据租户ID查询产品包详情
	 *
	 * @param tenantId 租户ID
	 */
	@IsAdministrator
	@GetMapping("/package-detail")
	@ApiOperationSupport(order = 13)
	@Operation(summary = "产品包详情", description = "传入tenantId")
	public R<TenantPackage> packageDetail(Long tenantId) {
		Tenant tenant = tenantService.getById(tenantId);
		return R.data(tenantPackageService.getById(tenant.getPackageId()));
	}

	/**
	 * 产品包配置
	 */
	@IsAdministrator
	@PostMapping("/package-setting")
	@ApiOperationSupport(order = 14)
	@Operation(summary = "产品包配置", description = "传入packageId")
	public R packageSetting(@Parameter(description = "租户ID", required = true) @RequestParam String tenantId, @Parameter(description = "产品包ID") Long packageId) {
		CacheUtil.evict(SYS_CACHE, TENANT_TENANT_ID, tenantId, Boolean.FALSE);
		CacheUtil.evict(SYS_CACHE, TENANT_PACKAGE_ID, tenantId, Boolean.FALSE);
		return R.status(tenantService.update(Wrappers.<Tenant>update().lambda().set(Tenant::getPackageId, packageId).eq(Tenant::getTenantId, tenantId)));
	}
}
